<?php

require_once 'config.php';

class Database
{
	private $db;

	function connect()
	{
		global $DBUSER, $DBPASS, $DBHOST, $DBDB;

		try
		{
			$this->db = new PDO("mysql:host=$DBHOST;dbname=$DBDB", $DBUSER, $DBPASS);
		}
		catch (PDOException $e)
		{
			die('connection error: ' . $e->getMessage());
		}

		$this->db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
	}

	function storeCampaign($campaign)
	{
		$this->db->prepare('INSERT INTO campaigns
			(id, name, description, start) VALUES (?, ?, ?, ?)
			ON DUPLICATE KEY UPDATE
				name = VALUES(name),
				description = VALUES(description),
				start = VALUES(start)')->execute(
			array(
				$campaign->id,
				$campaign->name,
				$campaign->description,
				$campaign->start,
			)
		);
	
		foreach ($campaign->getJobs() as $job)
		{
			$this->storeJob($campaign, $job);
		}
	}

	function storeJob($campaign, $job)
	{
		$this->db->prepare('INSERT INTO jobs
			(id, campaignId, name, description) VALUES (?, ?, ?, ?)
			ON DUPLICATE KEY UPDATE
				campaignId = VALUES(campaignId),
				name = VALUES(name),
				description = VALUES(description)')->execute(
			array(
				$job->id,
				$campaign->id,
				$job->name,
				$job->description,
			)
		);

		foreach ($job->getShifts() as $shift)
		{
			$this->storeShift($campaign, $job, $shift);
		}
	}

	function storeShift($campaign, $job, $shift)
	{
		$this->db->prepare('INSERT INTO shifts
			(id, campaignId, jobId, name, description, duration, start) VALUES (?, ?, ?, ?, ?, ?, FROM_UNIXTIME(?))
			ON DUPLICATE KEY UPDATE
				campaignId = VALUES(campaignId),
				jobId = VALUES(jobId),
				name = VALUES(name),
				description = VALUES(description),
				duration = VALUES(duration),
				start = VALUES(start)')->execute(
			array(
				$shift->id,
				$campaign->id,
				$job->id,
				$shift->name,
				$shift->description,
				$shift->duration,
				$shift->start,
			)
		);

		foreach ($shift->getHours() as $hour)
		{
			$this->storeHour($campaign, $job, $shift, $hour);
		}
	}

	function storeHour($campaign, $job, $shift, $hour)
	{
		$volunteer = $hour->getVolunteer();

		$this->db->prepare('INSERT INTO hours
			(id, campaignId, shiftId, volunteerId) VALUES (?, ?, ?, ?)
			ON DUPLICATE KEY UPDATE
				campaignId = VALUES(campaignId),
				shiftId = VALUES(shiftId),
				volunteerId = VALUES(volunteerId)')->execute(
			array(
				$hour->id,
				$campaign->id,
				$shift->id,
				$volunteer->id,
			)
		);

		$this->storeVolunteer($campaign, $volunteer);
	}

	function storeVolunteer($campaign, $volunteer)
	{
		$this->db->prepare('INSERT INTO volunteers
			(id, campaignId, name) VALUES (?, ?, ?)
			ON DUPLICATE KEY UPDATE
				campaignId = VALUES(campaignId),
				name = VALUES(name)')->execute(
			array(
				$volunteer->id,
				$campaign->id,
				$volunteer->name,
			)
		);
	}

	function deleteCampaign($id)
	{
		$this->db->prepare('DELETE FROM campaigns WHERE id = ?')->execute(array($id));
		$this->db->prepare('DELETE FROM jobs WHERE campaignId = ?')->execute(array($id));
		$this->db->prepare('DELETE FROM shifts WHERE campaignId = ?')->execute(array($id));
		$this->db->prepare('DELETE FROM hours WHERE campaignId = ?')->execute(array($id));
		$this->db->prepare('DELETE FROM volunteers WHERE campaignId = ?')->execute(array($id));
	}

	function getCampaignResults($id)
	{
		$sth = $this->db->prepare('SELECT hours.id as hoursId, volunteerId, shiftId, checked_in IS NOT NULL AS checked_in, hours.enabled AS enabled, hours.generated AS generated FROM volunteers JOIN hours ON (volunteers.id = hours.volunteerId) WHERE hours.campaignId = ?');
		$sth->execute(array($id));
		$sth->setFetchMode(PDO::FETCH_ASSOC);
		return $sth->fetchAll();
	}

	function getCampaigns()
	{
		$sth = $this->db->query('SELECT id, name, description, start FROM campaigns');
		$sth->setFetchMode(PDO::FETCH_ASSOC);
		return $sth->fetchAll();
	}

	function checkin($id, $time)
	{
		$this->db->prepare('UPDATE volunteers SET checked_in = FROM_UNIXTIME(?) WHERE id = ?')->execute(array($time, $id));;
	}

	function uncheckin($id)
	{
		$this->db->prepare('UPDATE volunteers SET checked_in = NULL WHERE id = ?')->execute(array($id));
	}

	function addToShift($volunteerId, $shiftId)
	{
		$id = genId();

		// Mine campaignId from shift
		$sth = $this->db->prepare('SELECT campaignId FROM shifts WHERE id = ?');
		$sth->execute(array($shiftId));
		$res = $sth->fetch();
		$campaignId = $res[0];

		$this->db->prepare('INSERT INTO hours (id, campaignId, volunteerId, shiftId, generated) VALUES (?, ?, ?, ?, 1)
			ON DUPLICATE KEY UPDATE enabled = 1')
			->execute(array($id, $campaignId, $volunteerId, $shiftId));
	}

	function setHourId($hourId, $volunteerId, $shiftId)
	{
		$this->db->prepare('UPDATE hours SET id = ?, generated = 0 WHERE volunteerId = ? AND shiftId = ?')
			->execute(array($hourId, $volunteerId, $shiftId));
	}

	function removeFromShift($volunteerId, $shiftId)
	{
		$this->db->prepare('UPDATE hours SET enabled = 0 WHERE volunteerId = ? AND shiftId = ?')
			->execute(array($volunteerId, $shiftId));
	}

	function getUpdates($time)
	{
		if (! $time)
			$time = 0;

		return array(
			'campaigns' => $this->getTableChanges('campaigns', $time),
			'jobs' => $this->getTableChanges('jobs', $time),
			'shifts' => $this->getTableChanges('shifts', $time),
			'hours' => $this->getTableChanges('hours', $time),
			'volunteers' => $this->getTableChanges('volunteers', $time)
			);
	}

	function getTableChanges($table, $time)
	{
		$extra = '';
		if ($table == 'volunteers')
			$extra = 'UNIX_TIMESTAMP(checked_in) AS checked_in,';
		else if ($table == 'shifts')
			$extra = 'UNIX_TIMESTAMP(start) AS start,';

		$sth = $this->db->prepare("SELECT *,
			$extra
			UNIX_TIMESTAMP(last_updated) AS last_updated
			FROM $table WHERE last_updated > FROM_UNIXTIME(?)
			ORDER BY last_updated ASC");
		$sth->execute(array($time));
		$sth->setFetchMode(PDO::FETCH_ASSOC);
		return $sth->fetchAll();
	}

	function streamEvents()
	{
		header('Content-Type: text/event-stream');
		header('Cache-Control: no-cache');
		echo "event: keepalive\n\n";
		ob_flush();
		flush();

		// seed with latest event
		$last_updated = max($this->db->query('SELECT UNIX_TIMESTAMP(MAX(last_updated)) FROM campaigns UNION
				SELECT UNIX_TIMESTAMP(MAX(last_updated)) FROM jobs UNION
				SELECT UNIX_TIMESTAMP(MAX(last_updated)) FROM shifts UNION
				SELECT UNIX_TIMESTAMP(MAX(last_updated)) FROM hours UNION
				SELECT UNIX_TIMESTAMP(MAX(last_updated)) FROM volunteers',
				PDO::FETCH_COLUMN, 0)->fetchAll());

		// only run for 30 seconds
		$start = time();
		while (time() - $start < 30)
		{
			sleep(5);

			$campaigns = $this->getTableChanges('campaigns', $last_updated);
			$jobs = $this->getTableChanges('jobs', $last_updated);
			$shifts = $this->getTableChanges('shifts', $last_updated);
			$hours = $this->getTableChanges('hours', $last_updated);
			$volunteers = $this->getTableChanges('volunteers', $last_updated);

			if ($campaigns || $jobs || $shifts || $hours || $volunteers)
			{
				$updates = array(
					'campaigns' => $campaigns,
					'jobs' => $jobs,
					'shifts' => $shifts,
					'hours' => $hours,
					'volunteers' => $volunteers,
				);

				echo "event: update\ndata: ", json_encode($updates), "\n\n";

				$last_updated = maxLastUpdated($updates);

				ob_flush();
				flush();
			}
		}
	}
}

function maxLastUpdated($v)
{
	if (empty($v))
		return 0;

	if (is_array($v) && !array_key_exists('last_updated', $v))
		return max(array_map('maxLastUpdated', $v));

	return $v['last_updated'];
}

function genId()
{
	return uniqid('@MD-');
}

?>
